﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DocuSignAPI.APIService;
using EnvelopeCreatorProject.Models.Recipients;
using EnvelopeCreatorProject.Models.Signers;
namespace EnvelopeCreatorProject.Models.Builders
{
    internal class EnvelopeRecipientBuilder : IEnvelopeBuilder
    {
        private readonly List<Recipient> _recipients;
        private readonly List<Tab> _tabs;
        private readonly List<Tab> _dateSignedFields;
        private readonly HostSigner _host;

        public EnvelopeRecipientBuilder(List<Recipient> recipients, List<Tab> tabs, List<Tab> dateSignedFields, HostSigner host)
        {
            _recipients = recipients;
            _tabs = tabs;
            _dateSignedFields = dateSignedFields;
            _host = host;
        }

        public void BuildTo(StringBuilder strBuilder)
        {
            int carbonCount = _recipients.FindAll((r) => r.Type == RecipientTypeCode.CarbonCopy).Count;
            int signerCount = _recipients.FindAll((r) => r.Type == RecipientTypeCode.Signer).Count;
            int inPersonCount = _recipients.FindAll((r) => r.Type == RecipientTypeCode.InPersonSigner).Count;
            int agent = _recipients.FindAll((r) => r.Type == RecipientTypeCode.Agent).Count;
            int certifiedDelivery = _recipients.FindAll((r) => r.Type == RecipientTypeCode.CertifiedDelivery).Count;
            int editors = _recipients.FindAll((r) => r.Type == RecipientTypeCode.Editor).Count;
            int intermediary = _recipients.FindAll((r) => r.Type == RecipientTypeCode.Intermediary).Count;

            var signTabs = _tabs.Where(t => t.Type == TabTypeCode.SignHere).ToList();
            var initialsTabs = _tabs.Where(t => t.Type == TabTypeCode.InitialHere).ToList();


            var signHandlers = new Dictionary<RecipientTypeCode, RecipientType>()
            {
                {RecipientTypeCode.CarbonCopy, new CarbonCopy(carbonCount)},
                {RecipientTypeCode.Signer, (new SignHereSigner(signerCount, signTabs, initialsTabs, _dateSignedFields))},
                {RecipientTypeCode.InPersonSigner, (new InPersonSigners(inPersonCount,signTabs, initialsTabs, _dateSignedFields, _host))},
                {RecipientTypeCode.Agent, new Agents(agent)},
                {RecipientTypeCode.CertifiedDelivery,new CertifiedDeliveries(certifiedDelivery)},
                {RecipientTypeCode.Editor, new Editors(editors)},
                {RecipientTypeCode.Intermediary, new Intermediaries(intermediary)}
            };

            _recipients.ForEach(r => signHandlers[r.Type].AddRecipientData(r));

            strBuilder.AppendLine("\"recipients\":{");
            strBuilder.AppendLine(String.Join(",", signHandlers.Where(item => item.Value.HasData()).Select(item => item.Value.GetRecipientsData())));
            strBuilder.AppendLine("}");
        }
    }
}
